--
-- Author: Li Yang
-- Date: 2016-06-23 19:33:11
--

local model_AStarQueue = import(".AStarQueue")

local AStar = class("AStar")

--[[ANode
	ANode = {
		x,--坐标
		y,
		block,--是否阻挡
		parentnode,--父节点
		nowCost,--成本值
	}

	struct SAround
	{
	    int x;
	    int y;
	    int f;
	};
	SAround aroundPos[8]={
	    {1,0,10},
	    {1,1,14},
	    {0,1,10},
	    {-1,1,14},
	    
	    {-1,0,10},
	    {-1,-1,14},
	    {0,-1,10},
	    {1,-1,14}
	};
]]


function AStar:ctor()
	--8个方向
	self.aroundPos = {
		{1,0,10},
	    {1,1,14},
	    {0,1,10},
	    {-1,1,14},
	    
	    {-1,0,10},
	    {-1,-1,14},
	    {0,-1,10},
	    {1,-1,14}
	}

	self.map_w = 0;
	self.map_h = 0;

	self.nodeVersion = 0;

	self.minF = 0xFFFFFFF;

	--[[
		地图数据
	]]
	self.m_map = {};

	--[[
		路径
	]]
	self.m_queue = {};

	self.path = {};
end

--[[设置地图数据
]]
function AStar:setMapData( data )
	self.m_map = {};
	local map_w = data.mapRow;
	local map_h = data.mapCol;
	for x=1,map_w do
		self.m_map[x] = {};
		for y=1,map_h do
			local ANode = {};
			ANode.x = x;
			ANode.y = y;
			ANode.block = data:IsBlockPos( x,y );
			self.m_map[x][y] = ANode;
		end
	end
	self.map_w = map_w;
	self.map_h = map_h;
	self.m_queue = nil;
end

function AStar:buildPath( startNode,endNode )
	self.path = {};
	table.insert(self.path,endNode)
	while endNode ~= startNode do
		endNode = endNode.parentnode;
		table.insert(self.path,endNode)
	end
	for i,v in ipairs(self.path) do
		print(i,v.x,v.y)
	end
end

--[[搜索路径
	params = {
		start_x,
		start_y,
		end_x,
		end_y
	}
]]
function AStar:searchPath( params )
	-- self.path = {};
	print("searchPath ==> 0:",params.start_x,params.start_y,params.end_x,params.end_y,self.map_w,self.map_h)
	if self.map_w <= params.end_x or self.map_h <= params.end_y or self.map_w <= params.start_x or self.map_h <= params.start_y or 0 >= params.end_x or 0 >= params.end_y or 0 >= params.start_x or 0 >= params.start_y then
		return;
	end
	print("searchPath ==> 1",params.start_x,params.start_y,params.end_x,params.end_y)
	params.start_x,params.start_y,params.end_x,params.end_y = self:reviseStartAndEndPos( params.start_x,params.start_y,params.end_x,params.end_y );
	print("searchPath ==> 2:",params.start_x,params.start_y,params.end_x,params.end_y)
	local startNode = self.m_map[params.start_x][params.start_y];
	local endNode = self.m_map[params.end_x][params.end_y];
	endNode.parentnode = nil;
	self.minF = 0xFFFFFFF;
	self.m_queue = model_AStarQueue.new();
	self.m_queue:reset(  );
	self.m_queue:push(startNode);
	startNode.parentnode = nil;
	startNode.nowCost = 0;
	startNode.version = self.nodeVersion;
	self.nodeVersion = self.nodeVersion + 1;
	local open = self.m_queue:pop();
	while open do
		if open.nowCost >= self.minF then
		
		else
			for i=1,8 do
				--方向值
				local pos = {};
				pos.x = self.aroundPos[i][1];
				pos.y = self.aroundPos[i][2];
				pos.f = self.aroundPos[i][3];
				local x = pos.x + open.x;
				local y = pos.y + open.y;
				if self:executeSearchLogic(x ,y ,pos,open ,endNode) then
					
				end
			end
		end
		open = self.m_queue:pop();
	end

	if endNode.parentnode then
		self:buildPath( startNode,endNode )
	end
end

--[[执行搜索逻辑
	搜索的位置
	x
	y
	pos 方向参数
	open 开启位置
]]
function AStar:executeSearchLogic(x ,y ,pos,open ,endNode)
	if x < 0 or y < 0 or x >= self.map_w or y >= self.map_h then
		return false;
	end
	
	local test = self.m_map[x][y];
	if not test then
		print("···",x,y)
	end
	local F = open.nowCost + pos.f;
	if test.block > 0 or F >= self.minF then
		return false;
	end
	if pos.f == 14 and (self:isBlock( self.m_map ,open.x + pos.x ,open.y ) or self:isBlock( self.m_map ,open.x ,open.y + pos.y )) then
		return false;
	end
	if test == endNode then
		endNode.parentnode = open;
		self.minF = F;
		return false;
	end
	if test.version == self.nodeVersion and test.nowCost <= F then
		return false;
	end

	test.parentnode = open;
	test.nowCost = F;
	test.version = self.nodeVersion;
	self.m_queue:push( test )
	return true;
end

function AStar:isBlock( map ,x ,y )
	if map[x][y].block > 0 then
		return true;
	end
	return false;
end


--[[
 检查开始点 和结束点
]]
function AStar:reviseStartAndEndPos( startX,startY,endX,endY )
	local endNode = self.m_map[endX][endY];
	local startNode = self.m_map[startX][startY];
	if endNode.block > Define_Map_Mark["cellMarkNone"] then
		endX,endY = self:findNonblockNearBlock( startX,startY,endX,endY  )
	end

	if startNode.block > Define_Map_Mark["cellMarkNone"] then
		startX,startY = self:findNonblockNearBlock( endX,endY,startX,startY  )
	end
	return startX,startY,endX,endY;
end

--[[发现附近空点
]]
function AStar:findNonblockNearBlock( startX,startY,endX,endY  )
	local vec_nodeList = {};
	local i = 0;
	while true do
		i = i + 1;
		local b = false;
		for y=endY - i, endy + i do
			for x=endX - i,endX + i do
				if self.map_w <= x or self.map_h <= y or 0 > x or 0 > y then

				else
					local temp_node = self.m_map[x][y];
					if temp_node.block == Define_Map_Mark["cellMarkNone"] then
						table.insert(vec_nodeList,temp_node)
						b = true;
					end
				end
			end
		end
		if b then
			break;
		end
	end

	local minNode = vec_nodeList[1];
	local min = math.abs(startX - minNode.x) + math.abs(startY - minNode.y);
	for i,v in ipairs(vec_nodeList) do
		local temNode = v;
		local dis = math.abs(startX - temNode.x) + math.abs(startY - temNode.y);
		if dis < min then
			minNode = temNode;
			min = dis;
		end
	end
	return minNode.x,minNode.y;
end



return AStar;