--
-- Author: liYang
-- Date: 2015-06-15 11:40:29
-- 45地图数据

local AStar_class =  import(".AStar")

local model_map = class("model_map")

Define_Map_Mark = {
	["cellMarkNone"] = 0,
	["cellMarkStop"] = 1,
	["cellMarkBuild"] = 2,
	["cellMarkRole"] = 3,
}

function model_map:ctor()
	--地图的大小 行 列
	self.mapRow = 96;
	self.mapCol = 96;
	--地图像素宽高
	self.pixelW = 0;
	self.pixelH = 0;
	--地图单位像素值
	self.tileW = 46;
	self.tileH = 34;
	
	--地图起始点坐标
	self.mapOrigin_x = 0;
	self.mapOrigin_y = 0;

	--当前显示的mapid
	self.ShowMapID = 1;

	--显示范围
	self.DisplayScope = {
		left = -1050,
		top = 760,
		right = 1050,
		bottom = -760
	};

	--[[碰撞数据
		examp
		[0][1] = {
			[1] = 1,--碰撞数据
			[2] = 0,--建筑数据
			[3] = 0,--角色数据
			[4] = 0,--保护数据
		}
	]]
	self.collisionLayerData = {};
	--地图当前缩放值
	self.currentScaleValuse = 1.0;

	self.m_Astar = AStar_class.new();
end

function model_map:InitMapData( params )
	self.m_Astar:setMapData( self )
end

--游戏坐标 转换为地图坐标
function model_map:getWorldToCell( x , y )
	local map_x = (x - self.mapOrigin_x) / self.tileW  + (self.mapOrigin_y - y) / self.tileH;
	local map_y = (self.mapOrigin_y - y) / self.tileH  - (x - self.mapOrigin_x) / self.tileW;
	-- print(self.mapOrigin_x,self.mapOrigin_y,x ,y,map_x,map_y)
	return math.floor(map_x) ,math.floor(map_y);
end

--游戏坐标 转换为地图坐标
function model_map:getWorldToCell_1( x , y )
	local map_x = (x - self.mapOrigin_x) / self.tileW  + (self.mapOrigin_y - y) / self.tileH;
	local map_y = (self.mapOrigin_y - y) / self.tileH  - (x - self.mapOrigin_x) / self.tileW;
	-- print(self.mapOrigin_x,self.mapOrigin_y,x ,y,map_x,map_y)
	return map_x ,map_y;
end

--地图坐标换算成游戏坐标
function model_map:getCellToWorld( x ,y )
	local word_x = (self.tileW / 2) * (x - y) + self.mapOrigin_x;
	local word_y = self.mapOrigin_y - (self.tileH / 2) * (x + y);
	-- print("model_map:getMapToWorldTransform",x,y,(self.tileW / 2) * (x - y),(self.tileH / 2) * (x + y),word_x ,word_y)
	return word_x ,word_y;
end

--将世界坐标转换成游戏画板坐标
function model_map:getWorldToNodeLayerSpace( x ,y )
	return x + self.pixelW / 2 - display.cx, y + self.pixelH / 2 - display.cy;
end

--设置地图缩放值
function model_map:setMapScale( scale )
	self.currentScaleValuse = scale;
	-- self.mapOrigin_x = self.mapOrigin_x * scale;
	-- self.mapOrigin_y = self.mapOrigin_y * scale;
	-- self.tileW = self.tileW * scale;
	-- self.tileH = self.tileH * scale;
end

--得到mapID
function model_map:getMapID(  )
	return self.ShowMapID;
end

--[[设置建筑数据
	layer 数据层
	x坐标
	y
	valuse 值
]]
function model_map:setMapData( params )
	self.collisionLayerData[params.x][params.y][params.layer] = params.valuse;
end

------------------------------------建筑相关--------------------------------------------------------

--[[设置地图数据通过建筑
	build 对应建筑
	dataType 数据类型 1 碰撞数据 2 建筑数据 3 角色数据 4 保护数据
	values 值 0 没有 1 有
]]
function model_map:setMapDataByBuild( build ,dataType ,values )
	local arenaInfo = GetBuildCfg(build.attribute_templateid)
	for x=build.cell_x,build.cell_x + arenaInfo.cell_w - 1  do
		for y=build.cell_y,build.cell_y + arenaInfo.cell_h - 1 do
			self.collisionLayerData[x][y][dataType] = values;
		end
	end
end

--[[设置地图数据通过角色
	soldier 对应建筑
	dataType 数据类型 1 碰撞数据 2 建筑数据 3 角色数据 4 保护数据
	values 值 0 没有 1 有
]]
function model_map:setMapDataBySoldier( soldier ,dataType ,values )
	local arenaInfo = GetSoldierSetup(soldier.attribute_templateid)
	for x=build.cell_x,build.cell_x + arenaInfo.cell_w - 1  do
		for y=build.cell_y,build.cell_y + arenaInfo.cell_h - 1 do
			self.collisionLayerData[x][y][dataType] = values;
		end
	end
end


--[[判断是否能修建建筑
]]
function model_map:IsCanConstructBuild( build )
	local arenaInfo = GetBuildCfg(build.attribute_templateid)
	for x=build.cell_x,build.cell_x + arenaInfo.cell_w - 1  do
		for y=build.cell_y,build.cell_y + arenaInfo.cell_h - 1 do
			if self.collisionLayerData[x][y][1] > 0 then
				return false;
			end
			if self.collisionLayerData[x][y][2] > 0 then
				return false;
			end
		end
	end
	return true;
end

--[[判断地图某点是否是阻挡点
]]
function model_map:IsBlockPos( x,y )
	for i,v in ipairs(self.collisionLayerData[x][y]) do
		if i == 1 and v > 0 then
			return Define_Map_Mark["cellMarkStop"];
		end

		if i == 2 and v > 0 then
			return Define_Map_Mark["cellMarkBuild"];
		end

		if i == 3 and v > 0 then
			return Define_Map_Mark["cellMarkRole"];
		end
	end
	return Define_Map_Mark["cellMarkNone"];
end

return model_map


