--
-- Author: LiYang
-- Date: 2016-01-09 17:24:08
-- 图块绘制参数数据

local mapData_TitleDrawInfo = class("mapData_TitleDrawInfo")
local class_unitTile = import(".unitTile")

function mapData_TitleDrawInfo:ctor()
	--[[当前地图信息显示标示位置
		设置这个可以以城市编号设置
		单位 以经纬度 -0.083664 0.121736
		比如重庆 标示位置解放碑中为标示位置
	]]
	self.CurrentMapPieceMarkLocation = {106.583611,29.563505};
	
	--绘制图块 宽 高
	self.draw_width = 8;
	self.draw_height = 8;

	--绘制的中心图块位置
	self.draw_Center = { x = 0, y = 0};

	--绘制中心经纬度
	self.draw_Center_LonAndLat = {lon = 0,lat = 0};

	--标示点像素信息
	self.markPointPixel = {x = 0,y = 0};

	--绘制点坐标
	self.drawTitlePixelLocation = {x = 0,y = 0}

	--缓存的图块信息
	self.cachePieceInfo = {};

	--记录的是标示位置
	self.recordPointPixel = {x = -10000,y = 0};

	--显示的比例
	self.displayScale = { x = 1 ,y = 1};

	self.displayScaleMark = { x = 1 , y = 1};

	self.displayZoom = 13;

	--检查地图更新长度
	self.checkPosLen = 200;

end

--[[初始图块绘制信息
	tile_w
	tile_h
	zoom 
]]
function mapData_TitleDrawInfo:initData( parame )
	self.draw_width = parame.tile_w;
	self.draw_height = parame.tile_h;
	self.displayZoom = parame.zoom;
	self.displayScaleMark.x ,self.displayScaleMark.y = managerMapDownloadInstance:getDownMarkByZoom( parame.zoom );
	--初始图块数据
	self:InitTitlePieceInfo();
	self.checkPosLen = (MAP_DATA.TileMarkInfo.UnitTitleWeith * 10 - 1024) / 2;
end

function mapData_TitleDrawInfo:InitTitlePieceInfo(  )
	for y=-self.draw_height / 2,self.draw_height / 2 do
        for x=-self.draw_width / 2,self.draw_width / 2 do
        	local titleData = class_unitTile.new();
        	titleData.currentstate = 1;
        	titleData.pos_x = x;
        	titleData.pos_y = y;
        	self:addPiece( titleData );
        	-- print(x,y ,titleData)
        end
    end
    print("InitTitlePieceInfo")
end


--[[更新 绘制地图块信息
	通过中心经纬度更新
	parame = {
		lon = 0,
		lat = 0,
	}
]]
function mapData_TitleDrawInfo:onUpdateDrawTitleInfo( parame )
	-- self.draw_Center_LonAndLat.lon = parame.lon;
	-- self.draw_Center_LonAndLat.lat = parame.lat;

	--设置标示点像素信息
	local lon_0 = parame.lon - self.CurrentMapPieceMarkLocation[1];
	local lat_0 = parame.lat - self.CurrentMapPieceMarkLocation[2];
	local x = lon_0 / managerMapDownloadInstance.PixelToLongitude;
	local y = lat_0 / managerMapDownloadInstance.PixelToLatitude;
	print("onUpdateDrawTitleInfo_0",parame.lon,parame.lat,x,y)
	self.draw_Center.x ,self.draw_Center.y = MAP_DATA.TileMarkInfo:getMapPiecePosByPixelCoordinate( x ,y );
	self.markPointPixel.x ,self.markPointPixel.y = MAP_DATA.TileMarkInfo:getPixelLocationByMapPos( self.draw_Center );
	self.drawTitlePixelLocation.x,self.drawTitlePixelLocation.y = self:getTitleMapDrawLocation();
	--得到中心位置的经纬度
	self.draw_Center_LonAndLat.lon,self.draw_Center_LonAndLat.lat = managerMapDownloadInstance:getMapLonByPixel( self.drawTitlePixelLocation.x,self.drawTitlePixelLocation.y )

	self.vectorType = 1;
	if self.draw_Center.y % 2 ~= 0 then
		self.vectorType = -1;
	end
	if self.vectorType ~= MAP_DATA.CurrentTileInfo.vectorType then
		MAP_DATA.CurrentTileInfo:SetTitleInfo( {
				scale = {x = self.displayScale.x ,y = self.displayScale.y},
				vector = self.vectorType,
			}  )
	end

	print("onUpdateDrawTitleInfo",self.markPointPixel.x ,self.markPointPixel.y,self.draw_Center.x ,self.draw_Center.y,self.draw_Center_LonAndLat.lon,self.draw_Center_LonAndLat.lat)
	
end

function mapData_TitleDrawInfo:setTitleDisplayScale( x , y )
	self.displayScale.x = x;
	self.displayScale.y = y;
end

--[[检查更新逻辑mapData_TitleDrawInfo
	相对中心点的像素坐标
	parame = {
		x = 0,
		y = 0,
		zoom = 13,
	}
]]
function mapData_TitleDrawInfo:onCheckUpdateLogic( params )
	
	local len = cc.pGetDistance({x = self.recordPointPixel.x,y = self.recordPointPixel.y},{ x = params.x , y = params.y});
	-- print("onCheckUpdateLogic",params.x , params.y,self.recordPointPixel.x,self.recordPointPixel.y,len)
	local ischangezoom = false;
	if self.displayZoom ~= params.zoom then
		ischangezoom = true;
		--得到最新显示比例
		self.displayScale.x = self.displayScaleMark.x / managerMapDownloadInstance.PixelToLongitude;
		self.displayScale.y = self.displayScaleMark.y / managerMapDownloadInstance.PixelToLatitude;
		MAP_DATA.TileMarkInfo:SetTitleInfo( {
				scale = {x = self.displayScale.x ,y = self.displayScale.y}
			}  )

		MAP_DATA.CurrentTileInfo:SetTitleInfo( {
				scale = {x = self.displayScale.x ,y = self.displayScale.y}
			}  )
		self.displayZoom = params.zoom;
		self.checkPosLen = (MAP_DATA.TileMarkInfo.UnitTitleWeith * 10 - 1024) / 2;

		self.recordPointPixel.x = params.x; 
		self.recordPointPixel.y = params.y; 
		local lon , lat = managerMapDownloadInstance:getCurrentShowCenterLongitudeLatitude(  );
		--更新信息
		self:onUpdateDrawTitleInfo( {
				lon = lon,
				lat = lat,
			} );
		printInfo("网路日志 %s","发送请求地图块数据 MSG_C2S_AREA_GET_INDEX".."x:".. (self.draw_Center.x - self.draw_width / 2) .."y:"..  (self.draw_Center.y - self.draw_height / 2))
		--发送玩家位置信息
	    gameTcp:SendMessage(MSG_C2S_AREA_GET_INDEX,{
	            self.draw_Center.x - self.draw_width / 2,
	            self.draw_Center.y - self.draw_height / 2,
	            self.draw_width + 1,
				self.draw_height + 1
	        })
	    return;
	end
	local isrequestNews = false;
	if len > self.checkPosLen then
		isrequestNews = true;
		print("onCheckUpdateLogic_1",params.x , params.y,self.recordPointPixel.x,self.recordPointPixel.y,len)

		self.recordPointPixel.x = params.x; 
		self.recordPointPixel.y = params.y; 
		local lon , lat = managerMapDownloadInstance:getCurrentShowCenterLongitudeLatitude(  );
		--更新信息
		self:onUpdateDrawTitleInfo( {
				lon = lon,
				lat = lat,
			} );
		printInfo("网路日志 %s","发送请求地图块数据 MSG_C2S_AREA_GET_INDEX =>".."x:".. (self.draw_Center.x - self.draw_width / 2) .."y:"..  (self.draw_Center.y - self.draw_height / 2))
		--发送玩家位置信息
	    gameTcp:SendMessage(MSG_C2S_AREA_GET_INDEX,{
	            self.draw_Center.x - self.draw_width / 2,
	            self.draw_Center.y - self.draw_height / 2,
	            self.draw_width + 1,
				self.draw_height + 1
	        })
	end
	-- print("onCheckUpdateLogic",isrequestNews,ischangezoom,self.displayZoom)
	-- if ischangezoom then
	-- 	--更新图块信息
	-- 	print("onCheckUpdateLogic",params.x , params.y,self.recordPointPixel.x,self.recordPointPixel.y,len,isrequestNews,ischangezoom,self.displayZoom)
	-- 	dispatchGlobaleEvent( "controler_mapPage_map_Title_layer" ,"updateshow",{ scalechange = ischangezoom });
	-- end
	
end

--[[得到地块图绘制位置
]]
function mapData_TitleDrawInfo:getTitleMapDrawLocation(  )
	--得到地块标示位置 相对于当前玩家标示位置的向量
	local vec_piexl_x ,vec_piexl_y = managerMapDownloadInstance:getMapPosByLonAndLat( self.CurrentMapPieceMarkLocation[1] , self.CurrentMapPieceMarkLocation[2] )
	print("getTitleMapDrawLocation",vec_piexl_x ,vec_piexl_y,self.markPointPixel.x,self.markPointPixel.y)
	return vec_piexl_x + self.markPointPixel.x,vec_piexl_y + self.markPointPixel.y;
end

function mapData_TitleDrawInfo:getCurrentPlayerPos(  )
	local vec_piexl_x = (MAIN_PLAYER.baseAttr._LoctionInfo.Longitude - self.CurrentMapPieceMarkLocation[1]) / managerMapDownloadInstance.PixelToLongitude;
	local vec_piexl_y = ( MAIN_PLAYER.baseAttr._LoctionInfo.Latitude - self.CurrentMapPieceMarkLocation[2]) / managerMapDownloadInstance.PixelToLatitude;
	local Tile_x ,Tile_y = MAP_DATA.TileMarkInfo:getMapPiecePosByPixelCoordinate( vec_piexl_x ,vec_piexl_y );
	print("getCurrentPlayerPos", MAIN_PLAYER.baseAttr._LoctionInfo.Longitude,MAIN_PLAYER.baseAttr._LoctionInfo.Latitude,Tile_x,Tile_y)
	return Tile_x ,Tile_y;
end

--[[得到地图块位置通过像素坐标
	得到的位置是相对于总标示位置的
]]
function mapData_TitleDrawInfo:getMapTilePosByPixelCoordinate( x ,y )
	local tile_x ,tile_y = MAP_DATA.TileMarkInfo:getMapPiecePosByPixelCoordinate( x , y );
	-- if conditions then
	-- 	--todo
	-- end
end

--添加图块信息
function mapData_TitleDrawInfo:addPiece( data )
	self.cachePieceInfo[data.pos_x.."_"..data.pos_y] = data;
end

function mapData_TitleDrawInfo:getPiece( index )
	return self.cachePieceInfo[index];
end


function mapData_TitleDrawInfo:getPieceByTilePos( x , y  )
	local pos_x = x - self.draw_Center.x;
	local pos_y = y - self.draw_Center.y;
	local index = pos_x.."_"..pos_y;
	return self.cachePieceInfo[index];
end

function mapData_TitleDrawInfo:getCenterDis( x ,y )
	local offset = cc.p(x - self.draw_Center.x,y - self.draw_Center.y);
	return cc.pGetLength(offset);
end

return mapData_TitleDrawInfo;