--
-- Author: LiYang
-- Date: 2016-01-09 17:24:08
-- 图块参数数据

local mapData_TitleInfo = class("mapData_TitleInfo")


function mapData_TitleInfo:ctor()

	self.UnitTitleWeith = 0;

	self.titleWeith = 100;--图块宽度

	self.DiagonalLen = 120;--对角线长度

	self.SideLen = 100;--边长
	
	self.pos = {};--6个点位置

	self.center = { x = 0, y = 0};

	--------------------渲染信息-------------------------
	--参考向量
	self.reference_vector = {x = 0,y = 0};

	self.vectorType = 1;

	self.constDirectionValuse = {
		{1,1},--奇数
		{0,1},--偶数
	};

	--显示的比例
	self.displayScale = { x = 1 ,y = 1};
end

function mapData_TitleInfo:initData( marksize)
	--标准尺寸
	self.markSize = marksize;
	self:SetTitleInfo( {
		scale = {x = 1 ,y = 1},
		vector = 1,
		}  )
end

--[[设置图块信息
	info = {
		scale = {x = 1 ,y = 1},
		vector = 1,(方向)
	}
]]
function mapData_TitleInfo:SetTitleInfo( info  )
	print("mapData_TitleInfo:SetTitleInfo")
	
	self.UnitTitleWeith = self.markSize;
	self.UnitTitleHeight = self.markSize;
	if info.vector then
		self.vectorType = info.vector;
	end
	
	self.reference_vector.x = self.UnitTitleWeith / 2 * self.vectorType;
	self.reference_vector.y = self.UnitTitleWeith * math.sin(math.rad(60));

	local titleWeith = self.markSize;
	local titleSideLen = titleWeith / 2 * math.tan(math.rad(30)) * 2;--边长
	local titleDiagonal = titleWeith / 2 / math.cos(math.rad(30));--对角线长度
	--逆时针旋转
	local pos_0 = { x = 0 ,y = titleDiagonal};
	local pos_1 = cc.pRotateByAngle(pos_0, {x = 0,y = 0}, math.rad(60));
	local pos_2 = cc.pRotateByAngle(pos_0, {x = 0,y = 0}, math.rad(120));
	local pos_3 =  { x = 0 ,y = -titleDiagonal};
	local pos_4 = cc.pRotateByAngle(pos_0, {x = 0,y = 0}, math.rad(240));
	local pos_5 = cc.pRotateByAngle(pos_0, {x = 0,y = 0}, math.rad(300));

	self.titleWeith = titleWeith;
	self.DiagonalLen = titleDiagonal;
	self.SideLen = titleSideLen;
	self.pos[1] = pos_0;
	self.pos[2] = pos_1;
	self.pos[3] = pos_2;
	self.pos[4] = pos_3;
	self.pos[5] = pos_4;
	self.pos[6] = pos_5;

	--根据比例值更新坐标
	self.displayScale.x = info.scale.x;
	self.displayScale.y = info.scale.y;
	self.reference_vector.x = self.reference_vector.x * info.scale.x;
	self.reference_vector.y = self.reference_vector.y * info.scale.y;
	self.UnitTitleWeith = self.UnitTitleWeith * info.scale.x;
	self.UnitTitleHeight = self.UnitTitleHeight * info.scale.y;
	self.titleWeith = titleWeith * info.scale.x;
	self.DiagonalLen = titleDiagonal * info.scale.x;
	self.SideLen = titleSideLen * info.scale.x;
	for i=1,6 do
		self.pos[i].x = self.pos[i].x * info.scale.x;
		self.pos[i].y = self.pos[i].y * info.scale.y;
	end
	print("mapData_TitleInfo:SetTitleInfo",self.displayScale.x,self.displayScale.y )
	print(titleSideLen,titleDiagonal,pos_0.x ,pos_0.y,pos_1.x ,pos_1.y ,pos_2.x ,pos_2.y,pos_3.x ,pos_3.y,pos_4.x ,pos_4.y,pos_5.x ,pos_5.y) 
end


--[[通过地图位置得到绘制位置
	return 返回的是相对于标示经纬度 的像素位置
]]
function mapData_TitleInfo:getPixelLocationByMapPos( pos )
	local dir = 1;
	if pos.y % 2 == 0 then
		dir = 2;
	end
	--得到当前高度的中心点
	local x = self.reference_vector.x * self.constDirectionValuse[dir][1];
	if pos.y == 0 then
		x = 0;
	end
	local y = self.reference_vector.y;
	return x + self.UnitTitleWeith * pos.x ,y * pos.y;
end


--[[通过像素坐标得到地图块位置
	(相对于标示位置的像素坐标)
	x,
	y
]]
function mapData_TitleInfo:getMapPiecePosByPixelCoordinate( x ,y )
	local handle_x = x;
	local handle_y = y;
	local pos_y = math.modf(handle_y / self.reference_vector.y);
	local pos_x = math.modf(handle_x / self.UnitTitleWeith);
	-- print("checkcenterPos",pos_x,pos_y)
	--判断具体位置
	for y1=pos_y - 1,pos_y + 1 do
		for x1=pos_x - 1,pos_x + 1 do
			local CheckResult = self:IsMapPieceContainPos( {
				mapPiecePos_x = x1,
				mapPiecePos_y = y1,
				x = x,
				y = y,
			} )
			if CheckResult then
				return x1 ,y1;
			end
		end
	end
end

--[[
	params = {
		mapPiecePos_x = 0,
		mapPiecePos_y = 0,
		x,
		y
	}
]]
function mapData_TitleInfo:IsMapPieceContainPos( params )
	--得到地图块位置信息
	local piece_x , piece_y = self:getPixelLocationByMapPos( { x = params.mapPiecePos_x,y = params.mapPiecePos_y} );
	local distance = cc.pGetDistance({x = piece_x,y = piece_y},{ x = params.x , y = params.y});

	if distance > self.DiagonalLen then
		return false;
	end
	-- print("IsMapPieceContainPos",distance,self.titleWeith / 2,piece_x,piece_y)
	if distance <= self.titleWeith / 2 then
		return true;
	end
	--判断是否相交
	local check_pos = { x = params.x - piece_x ,y = params.y - piece_y};
	-- print("IsMapPieceContainPos",check_pos.x ,check_pos.y)
	if cc.pIsSegmentIntersect(self.center,check_pos,self.pos[1],self.pos[2]) then
		-- print(1,2)
		return false;
	end
	-- print("====",check_pos.x,check_pos.y)
	if cc.pIsSegmentIntersect(self.center,check_pos,self.pos[2],self.pos[3]) then
		-- print(2,3)
		return false;
	end

	if cc.pIsSegmentIntersect(self.center,check_pos,self.pos[3],self.pos[4]) then
		-- print(3,4)
		return false;
	end

	if cc.pIsSegmentIntersect(self.center,check_pos,self.pos[4],self.pos[5]) then
		-- print(4,5)
		return false;
	end

	if cc.pIsSegmentIntersect(self.center,check_pos,self.pos[5],self.pos[6]) then
		-- print(5,6)
		return false;
	end

	if cc.pIsSegmentIntersect(self.center,check_pos,self.pos[6],self.pos[1]) then
		-- print(6,1)
		return false;
	end
	return true;
end

function mapData_TitleInfo.getInstance()
	if mapData_TitleInfo.instance == nil then
		mapData_TitleInfo.instance = mapData_TitleInfo.new()
	end

	return mapData_TitleInfo.instance
end

--MapTitleInfo = mapData_TitleInfo.getInstance();

return mapData_TitleInfo;