--
-- Author: Li yang
-- Date: 2015-11-04 14:04:14
-- 地图信息管理类

local model_mapManager = class("model_mapManager")

local class_unitTile = import(".unitTile")

local class_TileInfo = import(".TileInfo")

local class_mapData_TitleInfo = import(".mapData_TitleInfo");

local class_Cache = import(".mapData_Cache")

local class_mapData_TitleDrawInfo = import(".mapData_TitleDrawInfo");

--更新检查点
model_mapManager.CONST_CHECK_POS = {
	[1] = {-display.width/2,0},
	[2] = {-display.width/2,display.height/2},
	[3] = {0,display.height/2},
	[4] = {display.width/2,display.height/2},
	[5] = {display.width/2,0},
	[6] = {display.width/2,-display.height/2},
	[7] = {0,-display.height/2},
	[8] = {-display.width/2,-display.height/2},
	[9] = {0,0},
}

--地图显示大小比例
model_mapManager.CONST_ZOOM_RATION = {
	[12] = 2.5,
	[13] = 1,
	[14] = 0.5,
	[15] = 0.25,
	[16] = 0.1,
	[17] = 0.05,
}

--[[像素与经纬度比例
	经纬度/像素
	打时候要区分下zoom 下的具体比例
]]
model_mapManager.CONST_ZOOM_NORM = {
	PixelToLongitude = 0.000143878042697906494140625,
	PixelToLatitude = 0.000110837554931640625,
}

function model_mapManager:ctor()
	
	self.MapDisplayMarkLoction = { lon = 106.499947,lat = 29.685241}
	--总数
	self.TileSum = 0;

	--地图宽高
	self.Map_w = 1280;
	self.Map_h = 1280;

	--屏幕中心显示位置
	self.CurrentPos = { x = 0, y = 0};

	--屏幕像素大小
	self.Screen_w = 1280;
	self.Screen_h = 720;

	--地图实际显示宽高
	self.MapShow_w = 1280;
	self.MapShow_h = 1280;

	--地图显示比例
	self.Map_scale = 1;

	self.Map_zoom = 13;

	self.UnitTitleWeith = 60;

	--地图的一些参数信息
	self.mapConfig = {
		--地图缩放范围(在此范围内将地图分为100份)
		scalezoomscope = {14,14},--12 17 13,15
		mapscalescope = {1,1},--地图缩放范围
		unitscalechange = 0.5,--单位地图缩放改变值
		mapstartscale = 1,--地图初始缩放值
		markzoom = 14,--标示zoom
		markTileSize = 160,--标示图块大小
		tile_w = 8,
		tite_h = 8,
	}
	--是否是2点事件
	self.IsTwoTounchEvent = false;

	
	------------------------------图块功能相关参数--------------------------------------
	--得到缓存数据
	self.MapCacheData = class_Cache.new();

	--世界图块信息
	self.TileMarkInfo = class_mapData_TitleInfo.new();

	--当前图块信息
	self.CurrentTileInfo = class_mapData_TitleInfo.new();
	
	--图块绘制信息
	self.TileDrawInfo = class_mapData_TitleDrawInfo.new();

	--当前地图zoom 的缩放范围
	self.CurrentMapScaleScope = {min = 1 ,max = 1};

	self.lookatTileData = class_TileInfo.new();
end

--[[初始map数据

]]
function model_mapManager:initData(  )
	print("读取缓存地图数据")
	self.MapCacheData:ReadCacheData();
	print("初始化更新玩家位置")
	MAIN_PLAYER.baseAttr:UpdateLoctionInfo()
	print("初始化zoom")
	self.Map_zoom = self.mapConfig.markzoom;
	print("初始化图块信息:",self.mapConfig.markTileSize)
	self.TileMarkInfo:initData( self.mapConfig.markTileSize );--初始图块信息
	self.CurrentTileInfo:initData( self.mapConfig.markTileSize );
	--初始初始定位信息
	print("初始化地图下载数据")
	self:initDisplayLoctionLogic();
	--初始图块绘制信息
	print("初始化地图图块绘制信息")
	self.TileDrawInfo:initData({
		tile_w = self.mapConfig.tile_w,
		tile_h = self.mapConfig.tite_h,
		zoom = self.mapConfig.markzoom
		});
	print("初始化当前图块显示缩放值")
	self:setMapCurrentScaleScope( self.mapConfig.markzoom );
end

--[[初始显示位置逻辑
	1.根据玩家当前上线位置 取出保留位置的地图定位信息
	2.进行地图缓存逻辑
]]
function model_mapManager:initDisplayLoctionLogic()
	--得到缓存数据
	self.CurrentCacheMapCityCode = MAIN_PLAYER.baseAttr._LoctionInfo.CityCode;
	self.CurrentCacheMapData = self.MapCacheData:getSavePath( MAIN_PLAYER.baseAttr._LoctionInfo.CityCode );

	--暂时用玩家上线位置
	self.MapDisplayMarkLoction.lon = self.CurrentCacheMapData.lon--MAIN_PLAYER.baseAttr._LoctionInfo.Longitude;
	self.MapDisplayMarkLoction.lat = self.CurrentCacheMapData.lat--MAIN_PLAYER.baseAttr._LoctionInfo.Latitude;
	--更新地图下载信息
	managerMapDownloadInstance:setMapCenterMarkInfo(
        {
            center = {self.MapDisplayMarkLoction.lon,self.MapDisplayMarkLoction.lat},
            zoom = self.Map_zoom,
            path = self.CurrentCacheMapData.path,
        }
    )

    --设置玩家显示点
end

--返回玩家位置
function model_mapManager:returnPlayerPos(  )
	--更新地图位置
	local x ,y = managerMapDownloadInstance:getMapPosByLonAndLat( MAIN_PLAYER.baseAttr._LoctionInfo.Longitude,MAIN_PLAYER.baseAttr._LoctionInfo.Latitude );
	self.CurrentPos.x = -x;
	self.CurrentPos.y = -y;
	--更新位置
	mapSystemInstance._controlerMap.MapLayer:updateMapShow( self.CurrentPos , nil , nil);
end

function model_mapManager:setMapZoomScaleMark(  )

end

function model_mapManager:setMapPostion( pos )
	self.CurrentPos = pos;
end

function model_mapManager:getMapCurrentPostion(  )
	return self.CurrentPos;
end

--设置当前地图的缩放范围
function model_mapManager:setMapCurrentScaleScope( zoom )
	local maxZoom = zoom - 1;
	if maxZoom < self.mapConfig.scalezoomscope[1] then
		maxZoom = self.mapConfig.scalezoomscope[1];
	end
	local minZoom = zoom + 1;
	if minZoom > self.mapConfig.scalezoomscope[2] then
		minZoom = self.mapConfig.scalezoomscope[2];
	end
	self.CurrentMapScaleScope.min = self.CONST_ZOOM_RATION[zoom] / self.CONST_ZOOM_RATION[maxZoom]
	self.CurrentMapScaleScope.max = self.CONST_ZOOM_RATION[zoom] / self.CONST_ZOOM_RATION[minZoom]
	-- CallAndroidPrint( "model_mapManager:"..self.CurrentMapScaleScope.min..","..self.CurrentMapScaleScope.max)
end



--更新地图位置
function model_mapManager:updateMapPostion( pos )
	self.CurrentPos.x = self.CurrentPos.x + pos.x;
	self.CurrentPos.y = self.CurrentPos.y + pos.y;
	--更新显示
end

--更新地图缩放尺寸
function model_mapManager:updateMapScale( scale )
	--实际缩放值
	local realityScale = self.Map_scale + scale;
	-- CallAndroidPrint( "tounchesNumber TouchesAllMoveLogic change update:"..self.CurrentMapScaleScope.min..","..self.CurrentMapScaleScope.max..","..realityScale..",scale:"..scale)
	if realityScale < self.CurrentMapScaleScope.min then

		realityScale = self.CurrentMapScaleScope.min;
		--执行改变zoom操作
		if self.Map_zoom > self.mapConfig.scalezoomscope[1] then
			self.Map_zoom = self.Map_zoom - 1;
			self:setMapCurrentScaleScope( self.Map_zoom )
		end
		self.Map_scale = 1;
		return false;
	elseif realityScale > self.CurrentMapScaleScope.max then
		realityScale = self.CurrentMapScaleScope.max;
		if self.Map_zoom < self.mapConfig.scalezoomscope[2] then
			self.Map_zoom = self.Map_zoom + 1;
			self:setMapCurrentScaleScope( self.Map_zoom )
		end
		
		self.Map_scale = 1;
		return false;
	end
	self.Map_scale = realityScale;
	return true;
end

function model_mapManager:sendRequestTileInfo( pos_x , pos_y )
	printInfo("网路日志 %s","发送请求图块数据 MSG_C2S_AREA_GET_INFO".."x:".. pos_x .."y:".. pos_y)
	--发送玩家位置信息
    gameTcp:SendMessage(MSG_C2S_AREA_GET_INFO,{
            pos_x,
            pos_y
        })
end


return model_mapManager;
